@@ -109,7 +109,10 @@ class View(API):
         self._info = None
 
     def __repr__(self):
-        return self._tspl
+        if len(self._tspl) == 1:
+            return repr(list(self._tspl.values())[0])
+        else:
+            return "\n".join([f'{key} = {value!r}' for key, value in self._tspl.items()])
     
     def explain(self):
         return self._post("debug", json=self._tspl['value']).json()
@@ -184,12 +187,12 @@ class View(API):
                 data = resp.json()
                 if isinstance(data, list):
                     for evt in data:
-                        try:
-                            evt['start'] = pd.to_datetime(dt64(evt['start']), utc=True)
-                            evt['end'] = pd.to_datetime(dt64(evt['end']), utc=True)
-                        except:
-                            evt['start'] = pd.to_timedelta(dt64(evt['start']), utc=True)
-                            evt['end'] = pd.to_timedelta(dt64(evt['end']), utc=True)
+                        if type(evt['start']) is int:
+                            evt['start'] = pd.to_timedelta(evt['start'])
+                            evt['end'] = pd.to_timedelta(evt['end'])
+                        else:
+                            evt['start'] = pd.Timestamp(evt['start'])
+                            evt['end'] = pd.Timestamp(evt['end'])
                         if t != "event":
                             evt[name] = fromJSON(t, evt['value'])
                         if self._df:
