@@ -171,14 +171,19 @@ class Credentials(object):
 
     def __repr__(self):
         return "Credentials(auth_key='{}', host='{}')".format(
-            self.auth_key, self.host)
+            repr(self.auth_key), self.host)
 
 
 class API(object):
     def __init__(self, credentials, *prefix, params={}):
         self._credentials = credentials
-        self._session = requests.Session()
-        self._session.headers.update({ 'auth-key': credentials.auth_key })
+        if credentials.auth_key:
+            self._session = requests.Session()
+            for auth in credentials.auth_key.items():
+                self._session.auth = auth
+                break
+        else:
+            self._session = requests
         self._prefix = prefix
         self._params = params
 
@@ -256,22 +261,22 @@ class API(object):
             return resp
 
     def _get(self, *parts, params={}, headers={}):
-        return self._req(requests.get, parts, params, headers)
+        return self._req(self._session.get, parts, params, headers)
 
     def _put(self, *parts, params={}, headers={}, json={}):
-        return self._req(requests.put, parts, params, headers, data=json)
+        return self._req(self._session.put, parts, params, headers, data=json)
 
     def _post(self, *parts, params={}, headers={}, json={}, raw=False):
-        return self._req(requests.post, parts, params, headers, data=json, raw=raw)
+        return self._req(self._session.post, parts, params, headers, data=json, raw=raw)
 
     def _delete(self, *parts, params={}, headers={}):
-        return self._req(requests.delete, parts, params, headers)
+        return self._req(self._session.delete, parts, params, headers)
 
     def _patch(self, *parts, params={}, headers={}, json={}):
-        return self._req(requests.patch, parts, params, headers, data=json)
+        return self._req(self._session.patch, parts, params, headers, data=json)
 
     def _head(self, *parts, params={}, headers={}, json={}):
-        return self._req(requests.head, parts, params, headers)
+        return self._req(self._session.head, parts, params, headers)
 
 
 class SentenaiException(Exception): pass
